from pathlib import Path
import os
import subprocess


ASSET_BASE_URL = "https://github.com/google/heir/releases/download/nightly/"

def abort_cleanup(filename):
    cwd = Path(dir=os.getcwd())
    tmpfile = cwd / filename
    os.remove(tmpfile)


def load_nightly(filename) -> Path:
    """Fetches the nightly heir-opt binary from GitHub and returns the path to it."""
    print(f"Loading {filename} nightly binary")
    # TODO: how to clean up the tmpdir after ipython closes?
    # At worst, the user will see this in their heir_play dir and delete it.
    cwd = Path(dir=os.getcwd())
    tmpfile = cwd / filename
    if os.path.isfile(tmpfile):
        print(f"Using existing local {filename}")
        return tmpfile

    # -L follows redirects, necessary for GH asset downloads
    asset_url = ASSET_BASE_URL + filename
    proc = subprocess.run(["curl", "-L", "-o", tmpfile, asset_url])
    if proc.returncode != 0:
        print(f"Error downloading {filename}")
        print(proc.stderr)
        return None

    proc = subprocess.run(["chmod", "a+x", tmpfile])
    if proc.returncode != 0:
        print(f"Error modifying permissions on {filename}")
        print(proc.stderr)
        abort_cleanup(filename)
        return None

    return tmpfile



import os
import shlex
import subprocess

from IPython.core.magic import Magics
from IPython.core.magic import cell_magic
from IPython.core.magic import magics_class


class BinaryMagic(Magics):
    def __init__(self, shell, binary_path):
        """
        Initialize a magic for running a binary with a given path.
        """
        super(BinaryMagic, self).__init__(shell)
        self.binary_path = binary_path

    def run_binary(self, line, cell):
        """
        Run the binary on the input cell.

        Args:
            line: The options to pass to the binary.
            cell: The input to pass to the binary.
        """
        print(f"Running {self.binary_path}...")
        completed_process = subprocess.run(
            [os.path.abspath(self.binary_path)] + shlex.split(line),
            input=cell,
            text=True,
        )
        if completed_process.returncode != 0:
            print(f"Error running {self.binary_path}")
            print(completed_process.stdout)
            print(completed_process.stderr)
            return
        output = completed_process.stdout
        print(output)


@magics_class
class HeirOptMagic(BinaryMagic):
    def __init__(self, shell, binary_path="heir-opt"):
        """
        Initialize heir-opt with a path to the heir-opt binary.
        If not specified, will assume heir-opt is on the path.
        """
        super(HeirOptMagic, self).__init__(shell, binary_path)

    @cell_magic
    def heir_opt(self, line, cell):
        """
        Run heir-opt on the input cell.

        Args:
            line: The options to pass to heir-opt.
            cell: The input to pass to heir-opt.
        """
        return self.run_binary(line, cell)


@magics_class
class HeirTranslateMagic(BinaryMagic):
    def __init__(self, shell, binary_path="heir-translate"):
        """
        Initialize heir-translate with a path to the heir-translate binary.
        If not specified, will assume heir-translate is on the path.
        """
        super(HeirTranslateMagic, self).__init__(shell, binary_path)

    @cell_magic
    def heir_translate(self, line, cell):
        """
        Run heir-translate on the input cell.

        Args:
            line: The options to pass to heir-translate.
            cell: The input to pass to heir-translate.
        """
        return self.run_binary(line, cell)



"""A magic for running heir-opt nightly binary"""
__version__ = "0.0.1"

from .heir_opt import HeirOptMagic
from .heir_opt import HeirTranslateMagic
from .utils import load_nightly


def load_ipython_extension(ipython):
    ipython.register_magics(
        HeirOptMagic(ipython, binary_path=str(load_nightly("heir-opt")))
    )
    ipython.register_magics(
        HeirTranslateMagic(ipython, binary_path=str(load_nightly("heir-translate")))
    )
